__all__ = [
    'configure_app',
    'AddPanels',
]

import re

from docutils import nodes
from sphinx.transforms import SphinxTransform


def configure_app(app):
    app.add_transform(AddPanels)
    app.add_css_file('panels.css')
    app.add_js_file('panels.js')


class AddPanels(SphinxTransform):
    default_priority = 10

    # Mapping of name -> (default_visibile, icon)
    panel_defaults = {
        'prereq': (True, 'rocket'),
        'callout': (True, 'info-circle'),
        'challenge': (True, 'square-o'),
        'hiddenchallenge': (False, 'square-o'),
        'solution': (False, 'check-square-o'),
        'objectives': (True, 'line-chart'),
        'keypoints': (True, 'key'),
        'discussion': (False, 'bell'),
    }

    def apply(self, **kwargs):
        for node in self.document.traverse(nodes.Element)[::-1]:
            match = re.match(r'^ *{%\s*(\w+)\s*"([^"]+)"\s*%} *$', node.rawsource)
            if match:
                panel_type, title = match.groups()
                try:
                    visibile, icon = self.panel_defaults[panel_type]
                except KeyError:
                    raise ValueError(f'Unrecognised panel type {panel_type}',
                                     self.panel_defaults.keys())

                # Find the body of the panel
                inner_node = node
                current_nodes = []
                while True:
                    inner_node = inner_node.next_node(descend=False, siblings=True, ascend=False)
                    if inner_node is None:
                        raise ValueError(f'Failed to find end block for {node.rawsource} in {node.source}')
                    match = re.match(r'^ *{%\s*(\w+)\s*"([^"]+)"\s*%} *$', node.rawsource)
                    # Check if we're at the end of the panel block
                    if re.match(r'^\s*{%\s*end' + panel_type + r'\s*%}\s*$', inner_node.rawsource):
                        inner_node.parent.remove(inner_node)
                        break
                    current_nodes.append(inner_node)

                # Create a div
                panel_body = nodes.container()
                panel_body.attributes['classes'].append('panel-body')
                for inner_node in current_nodes:
                    inner_node.parent.remove(inner_node)
                    panel_body.append(inner_node)

                # Create the title text
                header_text = nodes.paragraph(ids=[title.replace(' ', '-').lower()])
                header_text.append(nodes.raw('', f'<i class="fa fa-{icon}"></i> ', format='html'))
                header_text.append(nodes.Text(title))

                # Create the title bar
                header = nodes.container()
                header.attributes['classes'].append('panel-header')
                if visibile:
                    header.attributes['classes'].append('open')
                header.append(header_text)

                # Move the inner nodes to the new container node and replace the parent
                new_node = nodes.container()
                new_node.attributes['classes'].append('panel')
                new_node.attributes['classes'].append('panel-' + panel_type)
                new_node.append(panel_body)
                new_node.insert(0, header)
                node.parent.replace(node, new_node)



from os.path import dirname

from recommonmark.transform import AutoStructify

from . import panels
from . import fix_markdown_file_downloads


extensions = [
    'sphinx_rtd_theme',
    'recommonmark',
    'sphinx.ext.mathjax',
    'nbsphinx',
]

templates_path = [
    '_templates',
]

html_theme = 'sphinx_rtd_theme'
html_show_sourcelink = True
html_theme_options = {
    'collapse_navigation': False,
}

exclude_patterns = [
    '**.ipynb_checkpoints',
]

html_context = {
    'display_github': True,
    'github_user': 'lhcb',
    'github_repo': 'starterkit-lessons',
    'github_version': 'master',
    'conf_py_path': '/source/',
}
highlight_language = 'none'

html_static_path = [
    f'{dirname(__file__)}/_static',
]

linkcheck_ignore = [
    # Expect certificate errors
    r'https://lhcb-portal-dirac\.cern\.ch/DIRAC/',
    r'https://lhcb-nightlies\.cern\.ch.*',
    # 404 if not logged in
    r'https://gitlab\.cern\.ch/.*/merge_requests/new',
    # Anchors to specific lines are generated with Javascript
    r'https://gitlab\.cern\.ch/lhcb/Stripping/blob/.*',
    # Seems to be unreliable?
    r'http://pdg.*\.lbl\.gov/.*',
    # FIXME: The URLs have changed
    r'https://research\.cs\.wisc\.edu/htcondor/.*',
]
linkcheck_workers = 32


def setup(app):
    app.add_config_value('recommonmark_config', {
        # 'url_resolver': lambda url: 'http://0.0.0.0/' + url,
        'auto_toc_tree_section': 'Contents',
        'enable_math': True,
        'enable_inline_math': True,
        'enable_eval_rst': True,
    }, True)
    app.add_transform(AutoStructify)
    fix_markdown_file_downloads.configure_app(app)
    panels.configure_app(app)
    for extra_setup_func in setup.extra_setup_funcs:
        extra_setup_func(app)


# Allow additional setup functions to be defined in projects
setup.extra_setup_funcs = []



__all__ = [
    'parse_args',
    'sphinx_config',
]

import argparse
import os
from os.path import join
import shutil
from subprocess import check_call, check_output

from . import sphinx_config

SOURCE_DIR = '.'
BUILD_DIR = 'build'


def parse_args():
    known_commands = {
        'clean': clean_docs,
        'build': build_docs,
        'check': check_docs,
        'deploy': deploy_docs,
    }

    parser = argparse.ArgumentParser()
    parser.add_argument('command', choices=known_commands)
    parser.add_argument('--source-dir', required=False, default=os.getcwd())
    parser.add_argument('--allow-warnings', action='store_true')

    args = parser.parse_args()

    known_commands[args.command](
        source_dir=args.source_dir,
        allow_warnings=args.allow_warnings
    )


def clean_docs(source_dir, allow_warnings=False):
    _sphinx_build('clean', source_dir, allow_warnings)


def build_docs(source_dir, allow_warnings=False):
    _sphinx_build('html', source_dir, allow_warnings)


def check_docs(source_dir, allow_warnings=False):
    _sphinx_build('linkcheck', source_dir, allow_warnings)


def deploy_docs(source_dir, allow_warnings=False):
    if os.environ['TRAVIS_BRANCH'] != 'master':
        print('This commit was made against', os.environ['TRAVIS_BRANCH'],
              'and not the master! No deploy!')
        return

    built_dir = join(source_dir, BUILD_DIR, 'html')
    git_rev = check_output(['git', 'rev-parse', '--short', 'HEAD'], cwd=source_dir)
    shutil.copy(join(source_dir, SOURCE_DIR, '.nojekyll'), built_dir)

    check_call(['git', 'init'], cwd=built_dir)
    check_call(['git', 'config', 'user.name', 'Alex Pearce'], cwd=built_dir)
    check_call(['git', 'config', 'user.email', 'alex@alexpearce.me'], cwd=built_dir)

    push_url = 'https://' + os.environ['GH_TOKEN'] + '@github.com/lhcb/starterkit-lessons.git'
    check_call(['git', 'remote', 'add', 'upstream', push_url], cwd=built_dir)
    check_call(['git', 'fetch', 'upstream'], cwd=built_dir)
    check_call(['git', 'reset', 'upstream/gh-pages'], cwd=built_dir)

    check_call(['touch', '.'], cwd=built_dir)

    check_call(['git', 'add', '-A', '.'], cwd=built_dir)
    check_call(['git', 'commit', '-m', 'Rebuild pages at ' + git_rev], cwd=built_dir)
    check_call(['git', 'push', '-q', 'upstream HEAD:gh-pages'], cwd=built_dir)


def _sphinx_build(cmd, source_dir, allow_warnings):
    cmd = ['sphinx-build', '-M', cmd, SOURCE_DIR, BUILD_DIR]
    if not allow_warnings:
        cmd += ['-W']
    return check_call(cmd, cwd=source_dir)



__all__ = [
    'configure_app',
    'FixMarkdownDownloads',
]

from os.path import isfile, join, dirname

from docutils import nodes
from sphinx.addnodes import download_reference, pending_xref
from sphinx.transforms import SphinxTransform


def configure_app(app):
    app.add_transform(FixMarkdownDownloads)


class FixMarkdownDownloads(SphinxTransform):
    default_priority = 5

    def apply(self, **kwargs):
        for old_node in self.document.traverse(pending_xref):
            if not isfile(join(dirname(old_node.source), old_node['reftarget'])):
                continue
            node1 = nodes.literal('', '', *old_node.children[0].children, classes=['xref', 'download'])
            old_node.replace(old_node.children[0], node1)
            new_node = download_reference(old_node.rawsource, '', *old_node.children, **old_node.attributes)
            old_node.parent.replace(old_node, new_node)



from setuptools import setup, find_packages
import os
from os.path import abspath, dirname, join
from io import open

here = abspath(dirname(__file__))

# Get the long description from the README file
with open(join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

package_data = []
for root, dirs, files in os.walk(join(here, 'src/starterkit_ci/sphinx_config/_static')):
    package_data += [join(root, fn) for fn in files]

setup(
    name='starterkit_ci',
    use_scm_version=True,
    description='Helpers for Starterkit Continuous Integration',
    long_description=long_description,
    long_description_content_type='text/markdown',

    url='https://github.com/lhcb/starterkit_ci',

    author='LHCb Starterkit',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords='Starterkit',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    python_requires='>=3.6',
    setup_requires=['setuptools_scm'],
    install_requires=[
        'sphinx',
        'sphinx-rtd-theme',
        'recommonmark',
        'nbsphinx',
    ],
    package_data={
        'starterkit_ci': package_data,
    },
    zip_safe=False,
    entry_points={
        'console_scripts': {
            'starterkit_ci = starterkit_ci:parse_args',
        }
    },
    project_urls={
        'Bug Reports': 'https://github.com/lhcb/starterkit_ci/issues',
        'Source': 'https://github.com/lhcb/starterkit_ci',
    },
)
